http_api_ktea = 'server'
local_exec_ktea = 'local_executable'
virtual_ktea = 'virtual_ktea'

FILE_PREFIX = "file"
SUPPLIER_PREFIX = "get"

pos = 'positive'
neg = 'negative'
rng = 'running'
brk = 'broken'
idl = 'idle'

TARGET_STANDARD = 'standard'
TARGET_INLIN = 'inline'
TARGET_STATE = 'state'
TARGET_PREFS = 'prefs'

TARGET_TYPES = [TARGET_STANDARD, TARGET_INLIN, TARGET_STATE, TARGET_PREFS]
DEFAULT_TARGET = TARGET_STANDARD

MAIN_WORKER = 'default'
TELEM_WORKER = 'telem'


statuses = [pos, neg, rng, idl]
